/*
COPYRIGHT (C) 2001  Stphane List (stephane.list@fr.alcove.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
*/


#include <linux/module.h>
#include <linux/init.h>

#include <asm/io.h>

#include <asm/rtai.h>
#include <rtai_fifos.h>
#include <rtai_sched.h>


#define RT_STACK_SIZE 1000
#define RT_PRIORITY 0
#define NO_FPU 0
#define FPU 1
 
#define TICK_PERIOD    100000    /*  100 usec = 100 *1000 ns  */
#define ONE_SHOT   

#define FIFO_0 0
#define FIFO_SIZE 1000

RT_TASK task;
long i=0;

void thread_code(int foo)
{
	while(1){
		if(i<10){
			i++;
			rtf_put(FIFO_0, &i, sizeof(int));
			rt_printk("Hello World : %d\n",i);		
			rt_task_wait_period();
		}
		rt_task_wait_period();
	}
}

int start_module(void)
{ 
        RTIME now,tick_period;

	rtf_create(FIFO_0,FIFO_SIZE);
	
#ifdef ONE_SHOT
	rt_set_oneshot_mode();
#endif
        if (rt_task_init(&task, thread_code, 0, RT_STACK_SIZE , RT_PRIORITY, NO_FPU, 0) < 0)
		printk("task init error\n");
		
	tick_period=start_rt_timer(nano2count(TICK_PERIOD));
	printk("tick_period = %ld\n",(long)tick_period);
	
	now=rt_get_time();
        if (rt_task_make_periodic(&task, now + 2*tick_period, tick_period) < 0)
		printk("task make periodic error\n");
	
	return 0;
}

void stop_module(void)
{
	stop_rt_timer();
	rtf_destroy(FIFO_0);
	rt_busy_sleep(10000000); // 10 ms
        if(rt_task_delete(&task)<0)
		printk("task delete error");
	
	printk("i= %ld\n",i);	 
}

module_init(start_module);
module_exit(stop_module);
