/*
COPYRIGHT (C) 2001  Stphane List (stephane.list@fr.alcove.com)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
*/


#include <linux/module.h>
#include <linux/init.h>

#include <asm/io.h>

#include <asm/rtai.h>
#include <rtai_shm.h>
#include <rtai_sched.h>

#define RT_STACK_SIZE 1000
#define RT_PRIORITY 0
#define NO_FPU 0
#define FPU 1
 
#define TICK_PERIOD    100000    /*  100 usec = 100 *1000 ns  */
#define ONE_SHOT   

#define MOUSE_IRQ 12


//#define GLOBAL_IRQ	/* != LINUX IRQ */

RT_TASK task;
long i=0;
long nb_mouse_interrupt=0;
long *adr_shm;

void thread_code(int foo)
{
	while(1){
		*adr_shm = nb_mouse_interrupt;
		rt_task_wait_period();
	}
}

static void handler(void)
{
	nb_mouse_interrupt++;
#ifdef GLOBAL_IRQ
	rt_pend_linux_irq(MOUSE_IRQ);
#endif
}

int start_module(void)
{ 
        RTIME now,tick_period;
#ifdef GLOBAL_IRQ
	rt_request_global_irq(MOUSE_IRQ,handler);
	rt_startup_irq(MOUSE_IRQ);
#else
	rt_request_linux_irq(MOUSE_IRQ,handler,"rt_mouse",handler);
#endif
	adr_shm = rtai_kmalloc(0xaaaa, sizeof(long));
	memset(adr_shm, 0x51, sizeof(long));
	
#ifdef ONE_SHOT
	rt_set_oneshot_mode();
#endif
        if (rt_task_init(&task, thread_code, 0, RT_STACK_SIZE , RT_PRIORITY, NO_FPU, 0) < 0)
		printk("task init error\n");
		
	tick_period=start_rt_timer(nano2count(TICK_PERIOD));
	printk("tick_period = %ld\n",(long)tick_period);
	
	now=rt_get_time();
        if (rt_task_make_periodic(&task, now + 2*tick_period, tick_period) < 0)
		printk("task make periodic error\n");
	
	return 0;
}

void stop_module(void)
{
#ifdef GLOBAL_IRQ
#warning Using GLOBAL IRQ
	rt_free_global_irq(MOUSE_IRQ);
#else
#warning Using LINUX IRQ
	rt_free_linux_irq(MOUSE_IRQ,handler);
#endif
	rtai_kfree(0xaaaa);
	stop_rt_timer();
	rt_busy_sleep(10000000); // 10 ms
        if(rt_task_delete(&task)<0)
		printk("task delete error");
	
}

module_init(start_module);
module_exit(stop_module);
